/*****************************************************************************
 *                        Copyright (c) 2001 Daniel Selman
 *                               Java Source
 *
 * This source is licensed under the GNU LGPL v2.1
 * Please read http://www.gnu.org/copyleft/lgpl.html for more information
 *
 ****************************************************************************/

package org.j3d.renderer.aviatrix3d.geom.hanim;

// External imports
import java.util.ArrayList;

// Local imports
import org.j3d.aviatrix3d.*;

import org.j3d.geom.hanim.HAnimHumanoid;
import org.j3d.geom.hanim.HAnimObject;

/**
 * Common AV3D implementation of the Humanoid object that uses GLSL shaders
 * to implement the mesh skinning algorithm.
 * <p>
 *
 * @author Justin Couch
 * @version $Revision: 1.1 $
 */
class ShaderHumanoid extends AVHumanoid
    implements NodeUpdateListener
{
    /** Group for containing the skin Shape nodes */
    private Group skinGroup;

    /** Geometry instances that implement BufferGeometry */
    private ArrayList bufferGeometry;

    /** Geometry instances that implement VertexGeometry */
    private ArrayList vertexGeometry;

    /** Collection of nodes that is the skin */
    private Node[] skin;

    /** The number of valid items in the skin list */
    private int numSkin;

    /**
     * An array for coordinates to be splatted to if the contained geometry
     * is an instance of VertexGeometry.
     */
    private float[] coordsCopy;

    /**
     * An array for coordinates to be splatted to if the contained geometry
     * is an instance of VertexGeometry.
     */
    private float[] normalsCopy;

    /**
     * Create a new, default instance of the site.
     */
    ShaderHumanoid()
    {
        bufferGeometry = new ArrayList();
        vertexGeometry = new ArrayList();

        skinGroup = new Group();
        addNode(skinGroup);
    }

    //----------------------------------------------------------
    // Methods defined by NodeUpdateListener
    //----------------------------------------------------------

    /**
     * Notification that its safe to update the node now with any operations
     * that could potentially effect the node's bounds.
     *
     * @param src The node or Node Component that is to be updated.
     */
    public void updateNodeBoundsChanges(Object src)
    {
        super.updateNodeBoundsChanges(src);
    }

    /**
     * Notification that its safe to update the node now with any operations
     * that only change the node's properties, but do not change the bounds.
     *
     * @param src The node or Node Component that is to be updated.
     */
    public void updateNodeDataChanges(Object src)
    {
        super.updateNodeDataChanges(src);
    }


    //----------------------------------------------------------
    // Methods defined by HAnimHumanoid
    //----------------------------------------------------------

    /**
     * All the skeletal changes are in for this frame, so update the matrix
     * values now. If nothing has changed, don't bother doing any calculations
     * and return immediately.
     */
    public void updateSkeleton()
    {
        super.updateSkeleton();
    }

    //----------------------------------------------------------
    // Methods defined by AVHumanoid
    //----------------------------------------------------------

    /**
     * Set the list of geometry that should be used by this humanoid. These are
     * not directly inserted into the scene graph, but are used for reference to
     * generate our own internal model. When skinned mesh animation takes place,
     * the coordinates are automatically fed from this humanoid to the appropriate
     * geometry. For best performance, the geometry should make use of
     * {@link org.j3d.renderer.aviatrix3d.geom.BufferGeometry} class.
     *
     * @param shapes List of representative shapes to use
     */
    public void setSkin(Node[] skins, int numSkins)
    {
        if(skinGroup.isLive())
        {
        }
        else
        {
            for(int i = 0; i < numSkins; i++)
            {
                Shape3D local_shape = (Shape3D)skinGroup.getChild(i);

                if(local_shape == null)
                {
                    local_shape = new Shape3D();
                    skinGroup.addChild(local_shape);
                }

                if(skins[i] instanceof Shape3D)
                {
                    Shape3D u_shape = (Shape3D)skins[i];
                    Appearance s_app = u_shape.getAppearance();
                    Geometry s_geom = u_shape.getGeometry();

                    Appearance l_app = local_shape.getAppearance();
                    if(l_app == null)
                    {
                        l_app = new Appearance();
                    }
                }
            }
        }
    }
}
